package com.logicbig.example;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.support.AbstractLobCreatingPreparedStatementCallback;
import org.springframework.jdbc.support.lob.DefaultLobHandler;
import org.springframework.jdbc.support.lob.LobCreator;
import org.springframework.jdbc.support.lob.LobHandler;
import org.springframework.stereotype.Repository;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;
import java.io.ByteArrayInputStream;
import java.io.Reader;
import java.io.StringReader;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

@Repository
public class ReportDao implements Dao<Report> {
    @Autowired
    private DataSource dataSource;
    private JdbcTemplate jdbcTemplate;

    @PostConstruct
    private void postConstruct() {
        jdbcTemplate = new JdbcTemplate(dataSource);
    }

    @Override
    public void save(Report report) {
        String sql = "insert into REPORT (NAME, CONTENT, IMAGE) values (?, ?, ?)";
        LobHandler lobHandler = new DefaultLobHandler();
        jdbcTemplate.execute(sql, new AbstractLobCreatingPreparedStatementCallback(lobHandler) {
            @Override
            protected void setValues(PreparedStatement ps, LobCreator lobCreator)
                    throws SQLException, DataAccessException {

                ps.setString(1, report.getName());
                Reader reader = new StringReader(report.getContent());
                lobCreator.setClobAsCharacterStream(ps, 2, reader,
                        report.getContent().length());

                ByteArrayInputStream inputStream = new ByteArrayInputStream(report.getImage());
                lobCreator.setBlobAsBinaryStream(ps, 3, inputStream, report.getImage().length);
            }
        });
    }

    @Override
    public Report load(long id) {
        List<Report> persons = jdbcTemplate.query("select * from Report where id =?",
                new Object[]{id}, (resultSet, i) -> {
                    return toReport(resultSet);
                });
        if (persons.size() == 1) {
            return persons.get(0);
        }
        throw new RuntimeException("No item found for id: " + id);
    }

    @Override
    public int delete(long id) {
        return 0; //todo
    }

    @Override
    public int update(Report customer) {
        return 0;//todo
    }

    @Override
    public List<Report> loadAll() {
        return jdbcTemplate.query("select * from REPORT", (resultSet, i) -> {
            return toReport(resultSet);
        });
    }

    private Report toReport(ResultSet resultSet) throws SQLException {
        Report report = new Report();
        report.setId(resultSet.getLong("ID"));
        report.setName(resultSet.getString("NAME"));

        LobHandler lobHandler = new DefaultLobHandler();
        String content = lobHandler.getClobAsString(resultSet, "CONTENT");
        report.setContent(content);

        byte[] image = lobHandler.getBlobAsBytes(resultSet, "IMAGE");
        report.setImage(image);

        return report;
    }
}